@@ -1,4 +1,7 @@
 #!/bin/bash
+
+set -euxo pipefail
+
 L10NDIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && cd .. && pwd )"
 SCRIPTDIR="$L10NDIR/bin"
 PODIR="$L10NDIR/pofiles"
@@ -45,5 +48,7 @@ iconv -t CP932 -o Japanese.nsh Japanese.nsh
 iconv -t CP1250 -o Slovak.nsh Slovak.nsh
 iconv -t CP1254 -o Turkish.nsh Turkish.nsh
 iconv -t CP1252 -o Norwegian.nsh Norwegian.nsh
-
+iconv -t CP1250 -o Polish.nsh Polish.nsh
+iconv -t CP852  -o Czech.nsh Czech.nsh
+#iconv -t CP852  -o Slovak.nsh Slovak.nsh
 
