@@ -20,12 +20,7 @@
 #include "progressdispatcher.h"
 #include "owncloudsetupwizard.h"
 #include "sharedialog.h"
-#if defined(Q_OS_MAC)
-#include "settingsdialogmac.h"
-#include "macwindow.h" // qtmacgoodies
-#else
 #include "settingsdialog.h"
-#endif
 #include "logger.h"
 #include "logbrowser.h"
 #include "account.h"
@@ -40,25 +35,26 @@
 #include <QMessageBox>
 #include <QDialog>
 #include <QHBoxLayout>
+#include <QScreen>
 
 #if defined(Q_OS_X11)
 #include <QX11Info>
 #endif
 
+#ifdef WITH_LIBCLOUDPROVIDERS
+#include "libcloudproviders/libcloudproviders.h"
+#endif
+
 namespace OCC {
 
 const char propertyAccountC[] = "oc_account";
 
 ownCloudGui::ownCloudGui(Application *parent)
     : QObject(parent)
-    , _tray(0)
-#if defined(Q_OS_MAC)
-    , _settingsDialog(new SettingsDialogMac(this))
-#else
+    , _tray(nullptr)
     , _settingsDialog(new SettingsDialog(this))
-#endif
-    , _logBrowser(0)
-    , _recentActionsMenu(0)
+    , _logBrowser(nullptr)
+    , _recentActionsMenu(nullptr)
     , _app(parent)
 {
     _tray = new Systray();
@@ -75,6 +71,12 @@ ownCloudGui::ownCloudGui(Application *parent)
 
     _tray->show();
 
+#ifdef WITH_LIBCLOUDPROVIDERS
+    auto exporter = new LibCloudProviders(this);
+    exporter->start();
+    connect(exporter, &LibCloudProviders::showSettings, this, &ownCloudGui::slotShowSettings);
+#endif
+
     ProgressDispatcher *pd = ProgressDispatcher::instance();
     connect(pd, &ProgressDispatcher::progressInfo, this,
         &ownCloudGui::slotUpdateProgress);
@@ -211,7 +213,7 @@ void ownCloudGui::slotComputeOverallSyncStatus()
     QVector<AccountStatePtr> problemAccounts;
     auto setStatusText = [&](const QString &text) {
         // Don't overwrite the status if we're currently syncing
-        if (FolderMan::instance()->currentSyncFolder())
+        if (FolderMan::instance()->isAnySyncRunning())
             return;
         _actionStatus->setText(text);
     };
@@ -814,9 +816,9 @@ void ownCloudGui::setupActions()
         _actionCrashFatal = new QAction("Crash now - qFatal", this);
         connect(_actionCrashFatal, &QAction::triggered, _app, &Application::slotCrashFatal);
     } else {
-        _actionCrash = 0;
-        _actionCrashEnforce = 0;
-        _actionCrashFatal = 0;
+        _actionCrash = nullptr;
+        _actionCrashEnforce = nullptr;
+        _actionCrashFatal = nullptr;
     }
 }
 
@@ -865,8 +867,8 @@ void ownCloudGui::slotUpdateProgress(const QString &folder, const ProgressInfo &
     }
 
     if (progress.totalSize() == 0) {
-        quint64 currentFile = progress.currentFile();
-        quint64 totalFileCount = qMax(progress.totalFiles(), currentFile);
+        qint64 currentFile = progress.currentFile();
+        qint64 totalFileCount = qMax(progress.totalFiles(), currentFile);
         QString msg;
         if (progress.trustEta()) {
             msg = tr("Syncing %1 of %2  (%3 left)")
@@ -1004,11 +1006,7 @@ void ownCloudGui::slotShowSettings()
 {
     if (_settingsDialog.isNull()) {
         _settingsDialog =
-#if defined(Q_OS_MAC)
-            new SettingsDialogMac(this);
-#else
             new SettingsDialog(this);
-#endif
         _settingsDialog->setAttribute(Qt::WA_DeleteOnClose, true);
         _settingsDialog->show();
     }
@@ -1063,16 +1061,12 @@ void ownCloudGui::slotHelp()
 
 void ownCloudGui::raiseDialog(QWidget *raiseWidget)
 {
-    if (raiseWidget && raiseWidget->parentWidget() == 0) {
+    if (raiseWidget && !raiseWidget->parentWidget()) {
         // Qt has a bug which causes parent-less dialogs to pop-under.
         raiseWidget->showNormal();
         raiseWidget->raise();
         raiseWidget->activateWindow();
 
-#if defined(Q_OS_MAC)
-        // viel hilft viel ;-)
-        MacWindow::bringToFront(raiseWidget);
-#endif
 #if defined(Q_OS_X11)
         WId wid = widget->winId();
         NETWM::init();
@@ -1125,16 +1119,13 @@ void ownCloudGui::slotShowShareDialog(const QString &sharePath, const QString &l
     //
     // The correct value will be found with a propfind from ShareDialog.
     // (we want to show the dialog directly, not wait for the propfind first)
-    SharePermissions maxSharingPermissions =
-        SharePermissionRead
-        | SharePermissionUpdate | SharePermissionCreate | SharePermissionDelete
-        | SharePermissionShare;
+    SharePermissions maxSharingPermissions = static_cast<SharePermissions>(accountState->account()->capabilities().defaultPermissions());
     if (!resharingAllowed) {
-        maxSharingPermissions = 0;
+        maxSharingPermissions = SharePermission(0);
     }
 
 
-    ShareDialog *w = 0;
+    ShareDialog *w = nullptr;
     if (_shareDialogs.contains(localPath) && _shareDialogs[localPath]) {
         qCInfo(lcApplication) << "Raising share dialog" << sharePath << localPath;
         w = _shareDialogs[localPath];
@@ -1162,8 +1153,7 @@ void ownCloudGui::slotRemoveDestroyedShareDialogs()
 
 void ownCloudGui::slotAbout()
 {
-    QString title = tr("About %1").arg(Theme::instance()->appNameGUI());
-    QString about = Theme::instance()->about();
+    const QString title = tr("About %1").arg(Theme::instance()->appNameGUI());
     QMessageBox *msgBox = new QMessageBox(this->_settingsDialog);
 #ifdef Q_OS_MAC
     // From Qt doc: "On macOS, the window title is ignored (as required by the macOS Guidelines)."
@@ -1174,12 +1164,9 @@ void ownCloudGui::slotAbout()
     msgBox->setAttribute(Qt::WA_DeleteOnClose, true);
     msgBox->setTextFormat(Qt::RichText);
     msgBox->setTextInteractionFlags(Qt::TextBrowserInteraction);
-    msgBox->setInformativeText("<qt>"+about+"</qt>");
+    msgBox->setInformativeText(QStringLiteral("<qt>%1</qt>").arg(Theme::instance()->about()));
     msgBox->setStandardButtons(QMessageBox::Ok);
-    QIcon appIcon = Theme::instance()->applicationIcon();
-    // Assume icon is always small enough to fit an about dialog?
-    qDebug() << appIcon.availableSizes().last();
-    msgBox->setIconPixmap(appIcon.pixmap(appIcon.availableSizes().last()));
+    msgBox->setIconPixmap(Theme::instance()->applicationIcon().pixmap(qApp->primaryScreen()->availableSize().width() / 4));
     msgBox->show();
 }
 
