@@ -49,11 +49,14 @@
 #include <QLocalSocket>
 #include <QStringBuilder>
 #include <QMessageBox>
-
+#include <QFileDialog>
 #include <QClipboard>
 
 #include <QStandardPaths>
 
+#ifdef Q_OS_MAC
+#include <CoreFoundation/CoreFoundation.h>
+#endif
 
 // This is the version that is returned when the client asks for the VERSION.
 // The first number should be changed if there is an incompatible change that breaks old clients.
@@ -192,8 +195,19 @@ SocketApi::SocketApi(QObject *parent)
         // Example for official signed packages: "9B5WD74GWJ." "com.owncloud.desktopclient" ".socketApi"
         socketPath = SOCKETAPI_TEAM_IDENTIFIER_PREFIX APPLICATION_REV_DOMAIN ".socketApi";
 #ifdef Q_OS_MAC
+        int ret = 0;
+        CFURLRef url = (CFURLRef)CFAutorelease((CFURLRef)CFBundleCopyBundleURL(CFBundleGetMainBundle()));
+        QString bundlePath = QUrl::fromCFURL(url).path();
+        QString cmd;
+
         // Tell Finder to use the Extension (checking it from System Preferences -> Extensions)
-        system("pluginkit -e use -i  " APPLICATION_REV_DOMAIN ".FinderSyncExt &");
+        cmd = QString("pluginkit -v -e use -i " APPLICATION_REV_DOMAIN ".FinderSyncExt");
+        ret = system(cmd.toLocal8Bit());
+
+        // Add it again. This was needed for Mojave to trigger a load.
+        cmd = QString("pluginkit -v -a ") + bundlePath + "Contents/PlugIns/FinderSyncExt.appex/";
+        ret = system(cmd.toLocal8Bit());
+
 #endif
     } else if (Utility::isLinux() || Utility::isBSD()) {
         QString runtimeDir;
@@ -232,11 +246,6 @@ SocketApi::~SocketApi()
     // All remaining sockets will be destroyed with _localServer, their parent
     ASSERT(_listeners.isEmpty() || _listeners.first().socket->parent() == &_localServer);
     _listeners.clear();
-
-#ifdef Q_OS_MAC
-    // Unload the extension (uncheck from System Preferences -> Extensions)
-    system("pluginkit -e ignore -i  " APPLICATION_REV_DOMAIN ".FinderSyncExt &");
-#endif
 }
 
 void SocketApi::slotNewConnection()
@@ -484,18 +493,19 @@ class GetOrCreatePublicLinkShare : public QObject
 {
     Q_OBJECT
 public:
-    GetOrCreatePublicLinkShare(const AccountPtr &account, const QString &localFile,
-        std::function<void(const QString &link)> targetFun, QObject *parent)
+    GetOrCreatePublicLinkShare(const AccountPtr &account,
+        const QString &serverPath, QObject *parent)
         : QObject(parent)
         , _account(account)
         , _shareManager(account)
-        , _localFile(localFile)
-        , _targetFun(targetFun)
+        , _serverPath(serverPath)
     {
         connect(&_shareManager, &ShareManager::sharesFetched,
             this, &GetOrCreatePublicLinkShare::sharesFetched);
         connect(&_shareManager, &ShareManager::linkShareCreated,
             this, &GetOrCreatePublicLinkShare::linkShareCreated);
+        connect(&_shareManager, &ShareManager::linkShareCreationForbidden,
+            this, &GetOrCreatePublicLinkShare::linkShareCreationForbidden);
         connect(&_shareManager, &ShareManager::serverError,
             this, &GetOrCreatePublicLinkShare::serverError);
     }
@@ -503,7 +513,7 @@ public:
     void run()
     {
         qCDebug(lcPublicLink) << "Fetching shares";
-        _shareManager.fetchShares(_localFile);
+        _shareManager.fetchShares(_serverPath);
     }
 
 private slots:
@@ -512,8 +522,11 @@ private slots:
         auto shareName = SocketApi::tr("Context menu share");
 
         // If shares will expire, create a new one every day.
+        QDate expireDate;
         if (_account->capabilities().sharePublicLinkDefaultExpire()) {
             shareName = SocketApi::tr("Context menu share %1").arg(QDate::currentDate().toString(Qt::ISODate));
+            expireDate = QDate::currentDate().addDays(
+                    _account->capabilities().sharePublicLinkDefaultExpireDateDays());
         }
 
         // If there already is a context menu share, reuse it
@@ -530,7 +543,8 @@ private slots:
 
         // otherwise create a new one
         qCDebug(lcPublicLink) << "Creating new share";
-        _shareManager.createLinkShare(_localFile, shareName, QString());
+        QString noPassword;
+        _shareManager.createLinkShare(_serverPath, shareName, noPassword, expireDate);
     }
 
     void linkShareCreated(const QSharedPointer<LinkShare> &share)
@@ -539,29 +553,34 @@ private slots:
         success(share->getLink().toString());
     }
 
+    void linkShareCreationForbidden(const QString &message)
+    {
+        qCInfo(lcPublicLink) << "Could not create link share:" << message;
+        emit error(message);
+        deleteLater();
+    }
+
     void serverError(int code, const QString &message)
     {
         qCWarning(lcPublicLink) << "Share fetch/create error" << code << message;
-        QMessageBox::warning(
-            0,
-            tr("Sharing error"),
-            tr("Could not retrieve or create the public link share. Error:\n\n%1").arg(message),
-            QMessageBox::Ok,
-            QMessageBox::NoButton);
+        emit error(message);
         deleteLater();
     }
 
+signals:
+    void done(const QString &link);
+    void error(const QString &message);
+
 private:
     void success(const QString &link)
     {
-        _targetFun(link);
+        emit done(link);
         deleteLater();
     }
 
     AccountPtr _account;
     ShareManager _shareManager;
-    QString _localFile;
-    std::function<void(const QString &url)> _targetFun;
+    QString _serverPath;
 };
 
 #else
@@ -570,14 +589,16 @@ class GetOrCreatePublicLinkShare : public QObject
 {
     Q_OBJECT
 public:
-    GetOrCreatePublicLinkShare(const AccountPtr &, const QString &,
-        std::function<void(const QString &link)>, QObject *)
+    GetOrCreatePublicLinkShare(const AccountPtr &, const QString &, QObject *)
     {
     }
 
     void run()
     {
     }
+signals:
+    void done(const QString &link);
+    void error(int code, const QString &message);
 };
 
 #endif
@@ -589,7 +610,11 @@ void SocketApi::command_COPY_PUBLIC_LINK(const QString &localFile, SocketListene
         return;
 
     AccountPtr account = fileData.folder->accountState()->account();
-    auto job = new GetOrCreatePublicLinkShare(account, fileData.serverRelativePath, [](const QString &url) { copyUrlToClipboard(url); }, this);
+    auto job = new GetOrCreatePublicLinkShare(account, fileData.serverRelativePath, this);
+    connect(job, &GetOrCreatePublicLinkShare::done, this,
+            [](const QString &url) { copyUrlToClipboard(url); });
+    connect(job, &GetOrCreatePublicLinkShare::error, this,
+            [=]() { emit shareCommandReceived(fileData.serverRelativePath, fileData.localPath, ShareDialogStartPage::PublicLinks); });
     job->run();
 }
 
@@ -629,60 +654,119 @@ void SocketApi::command_OPEN_PRIVATE_LINK(const QString &localFile, SocketListen
     fetchPrivateLinkUrlHelper(localFile, &SocketApi::openPrivateLink);
 }
 
+void SocketApi::command_OPEN_PRIVATE_LINK_VERSIONS(const QString &localFile, SocketListener *)
+{
+    auto openVersionsLink = [](const QString &link) {
+        QUrl url(link);
+        QUrlQuery query(url);
+        query.addQueryItem(QStringLiteral("details"), QStringLiteral("versionsTabView"));
+        url.setQuery(query);
+        Utility::openBrowser(url, nullptr);
+    };
+    fetchPrivateLinkUrlHelper(localFile, openVersionsLink);
+}
+
 void SocketApi::copyUrlToClipboard(const QString &link)
 {
     QApplication::clipboard()->setText(link);
 }
 
-void SocketApi::command_DOWNLOAD_VIRTUAL_FILE(const QString &filesArg, SocketListener *)
+void SocketApi::command_MAKE_AVAILABLE_LOCALLY(const QString &filesArg, SocketListener *)
 {
     QStringList files = filesArg.split(QLatin1Char('\x1e')); // Record Separator
-    auto suffix = QStringLiteral(APPLICATION_DOTVIRTUALFILE_SUFFIX);
 
     for (const auto &file : files) {
-        if (!file.endsWith(suffix) && !QFileInfo(file).isDir())
+        auto data = FileData::get(file);
+        if (!data.folder)
             continue;
-        QString relativePath;
-        auto folder = FolderMan::instance()->folderForPath(file, &relativePath);
-        if (folder) {
-            folder->downloadVirtualFile(relativePath);
-        }
+
+        // Update the pin state on all items
+        data.folder->vfs().setPinState(data.folderRelativePath, PinState::AlwaysLocal);
+
+        // Trigger sync
+        data.folder->schedulePathForLocalDiscovery(data.folderRelativePath);
+        data.folder->scheduleThisFolderSoon();
     }
 }
 
-/* Go over all the files ans replace them by a virtual file */
-void SocketApi::command_REPLACE_VIRTUAL_FILE(const QString &filesArg, SocketListener *)
+/* Go over all the files and replace them by a virtual file */
+void SocketApi::command_MAKE_ONLINE_ONLY(const QString &filesArg, SocketListener *)
 {
     QStringList files = filesArg.split(QLatin1Char('\x1e')); // Record Separator
-    auto suffix = QStringLiteral(APPLICATION_DOTVIRTUALFILE_SUFFIX);
 
     for (const auto &file : files) {
-        QString relativePath;
-        auto folder = FolderMan::instance()->folderForPath(file, &relativePath);
-        if (!folder)
+        auto data = FileData::get(file);
+        if (!data.folder)
             continue;
-        if (file.endsWith(suffix))
-            continue;
-        QFileInfo fi(file);
-        if (fi.isDir()) {
-            folder->journalDb()->getFilesBelowPath(relativePath.toUtf8(), [&](const SyncJournalFileRecord &rec) {
-                if (rec._type != ItemTypeFile || rec._path.endsWith(APPLICATION_DOTVIRTUALFILE_SUFFIX))
-                    return;
-                QString file = folder->path() + '/' + QString::fromUtf8(rec._path);
-                if (!FileSystem::rename(file, file + suffix)) {
-                    qCWarning(lcSocketApi) << "Unable to rename " << file;
-                }
-            });
-        } else {
-            SyncJournalFileRecord record;
-            if (!folder->journalDb()->getFileRecord(relativePath, &record) || !record.isValid())
-                continue;
-            if (!FileSystem::rename(file, file + suffix)) {
-                qCWarning(lcSocketApi) << "Unable to rename " << file;
-            }
-        }
-        folder->slotWatchedPathChanged(file); // make sure it is in the _localDiscoveryTracker list
-        FolderMan::instance()->scheduleFolder(folder);
+
+        // Update the pin state on all items
+        data.folder->vfs().setPinState(data.folderRelativePath, PinState::OnlineOnly);
+
+        // Trigger sync
+        data.folder->schedulePathForLocalDiscovery(data.folderRelativePath);
+        data.folder->scheduleThisFolderSoon();
+    }
+}
+
+void SocketApi::command_DELETE_ITEM(const QString &localFile, SocketListener *)
+{
+    QFileInfo info(localFile);
+
+    auto result = QMessageBox::question(
+        nullptr, tr("Confirm deletion"),
+        info.isDir()
+            ? tr("Do you want to delete the directory <i>%1</i> and all its contents permanently?").arg(info.dir().dirName())
+            : tr("Do you want to delete the file <i>%1</i> permanently?").arg(info.fileName()),
+        QMessageBox::Yes, QMessageBox::No);
+    if (result != QMessageBox::Yes)
+        return;
+
+    if (info.isDir()) {
+        FileSystem::removeRecursively(localFile);
+    } else {
+        QFile(localFile).remove();
+    }
+}
+
+void SocketApi::command_MOVE_ITEM(const QString &localFile, SocketListener *)
+{
+    auto fileData = FileData::get(localFile);
+    auto parentDir = fileData.parentFolder();
+    if (!fileData.folder)
+        return; // should not have shown menu item
+
+    QString defaultDirAndName = fileData.folderRelativePath;
+
+    // If it's a conflict, we want to save it under the base name by default
+    if (Utility::isConflictFile(defaultDirAndName)) {
+        defaultDirAndName = fileData.folder->journalDb()->conflictFileBaseName(fileData.folderRelativePath.toUtf8());
+    }
+
+    // If the parent doesn't accept new files, go to the root of the sync folder
+    QFileInfo fileInfo(localFile);
+    auto parentRecord = parentDir.journalRecord();
+    if ((fileInfo.isFile() && !parentRecord._remotePerm.hasPermission(RemotePermissions::CanAddFile))
+        || (fileInfo.isDir() && !parentRecord._remotePerm.hasPermission(RemotePermissions::CanAddSubDirectories))) {
+        defaultDirAndName = QFileInfo(defaultDirAndName).fileName();
+    }
+
+    // Add back the folder path
+    defaultDirAndName = QDir(fileData.folder->path()).filePath(defaultDirAndName);
+
+    auto target = QFileDialog::getSaveFileName(
+        nullptr,
+        tr("Select new location..."),
+        defaultDirAndName,
+        QString(), nullptr, QFileDialog::HideNameFilterDetails);
+    if (target.isEmpty())
+        return;
+
+    QString error;
+    if (!FileSystem::uncheckedRenameReplace(localFile, target, &error)) {
+        qCWarning(lcSocketApi) << "Rename error:" << error;
+        QMessageBox::warning(
+            nullptr, tr("Error"),
+            tr("Moving file failed:\n\n%1").arg(error));
     }
 }
 
@@ -691,7 +775,7 @@ void SocketApi::emailPrivateLink(const QString &link)
     Utility::openEmailComposer(
         tr("I shared something with you"),
         link,
-        0);
+        nullptr);
 }
 
 void OCC::SocketApi::openPrivateLink(const QString &link)
@@ -730,7 +814,8 @@ void SocketApi::sendSharingContextMenuOptions(const FileData &fileData, SocketLi
     // If sharing is globally disabled, do not show any sharing entries.
     // If there is no permission to share for this file, add a disabled entry saying so
     if (isOnTheServer && !record._remotePerm.isNull() && !record._remotePerm.hasPermission(RemotePermissions::CanReshare)) {
-        listener->sendMessage(QLatin1String("MENU_ITEM:DISABLED:d:") + tr("Resharing this file is not allowed"));
+        listener->sendMessage(QLatin1String("MENU_ITEM:DISABLED:d:") + (!record.isDirectory()
+            ? tr("Resharing this file is not allowed") : tr("Resharing this folder is not allowed")));
     } else {
         listener->sendMessage(QLatin1String("MENU_ITEM:SHARE") + flagString + tr("Share..."));
 
@@ -775,6 +860,16 @@ SocketApi::FileData SocketApi::FileData::get(const QString &localFile)
     return data;
 }
 
+QString SocketApi::FileData::folderRelativePathNoVfsSuffix() const
+{
+    auto result = folderRelativePath;
+    QString virtualFileExt = QStringLiteral(APPLICATION_DOTVIRTUALFILE_SUFFIX);
+    if (result.endsWith(virtualFileExt)) {
+        result.chop(virtualFileExt.size());
+    }
+    return result;
+}
+
 SyncFileStatus SocketApi::FileData::syncFileStatus() const
 {
     if (!folder)
@@ -791,6 +886,11 @@ SyncJournalFileRecord SocketApi::FileData::journalRecord() const
     return record;
 }
 
+SocketApi::FileData SocketApi::FileData::parentFolder() const
+{
+    return FileData::get(QFileInfo(localPath).dir().path().toUtf8());
+}
+
 void SocketApi::command_GET_MENU_ITEMS(const QString &argument, OCC::SocketListener *listener)
 {
     listener->sendMessage(QString("GET_MENU_ITEMS:BEGIN"));
@@ -814,36 +914,131 @@ void SocketApi::command_GET_MENU_ITEMS(const QString &argument, OCC::SocketListe
     // Some options only show for single files
     if (files.size() == 1) {
         FileData fileData = FileData::get(files.first());
-        bool isOnTheServer = fileData.journalRecord().isValid();
+        auto record = fileData.journalRecord();
+        bool isOnTheServer = record.isValid();
         auto flagString = isOnTheServer ? QLatin1String("::") : QLatin1String(":d:");
 
         if (fileData.folder && fileData.folder->accountState()->isConnected()) {
             sendSharingContextMenuOptions(fileData, listener);
             listener->sendMessage(QLatin1String("MENU_ITEM:OPEN_PRIVATE_LINK") + flagString + tr("Open in browser"));
+
+            // Add link to versions pane if possible
+            auto &capabilities = folder->accountState()->account()->capabilities();
+            if (capabilities.versioningEnabled()
+                && capabilities.privateLinkDetailsParamAvailable()
+                && isOnTheServer
+                && !record.isDirectory()) {
+                listener->sendMessage(QLatin1String("MENU_ITEM:OPEN_PRIVATE_LINK_VERSIONS") + flagString + tr("Show file versions in browser"));
+            }
+
+            // Conflict files get conflict resolution actions
+            bool isConflict = Utility::isConflictFile(fileData.folderRelativePath);
+            if (isConflict || !isOnTheServer) {
+                // Check whether this new file is in a read-only directory
+                QFileInfo fileInfo(fileData.localPath);
+                auto parentDir = fileData.parentFolder();
+                auto parentRecord = parentDir.journalRecord();
+                bool canAddToDir =  parentRecord._remotePerm.isNull()
+                    || (fileInfo.isFile() && !parentRecord._remotePerm.hasPermission(RemotePermissions::CanAddFile))
+                    || (fileInfo.isDir() && !parentRecord._remotePerm.hasPermission(RemotePermissions::CanAddSubDirectories));
+                bool canChangeFile =
+                    !isOnTheServer
+                    || (record._remotePerm.hasPermission(RemotePermissions::CanDelete)
+                           && record._remotePerm.hasPermission(RemotePermissions::CanMove)
+                           && record._remotePerm.hasPermission(RemotePermissions::CanRename));
+
+                if (isConflict && canChangeFile) {
+                    if (canAddToDir) {
+                        if (isOnTheServer) {
+                            // Conflict file that is already uploaded
+                            listener->sendMessage(QLatin1String("MENU_ITEM:MOVE_ITEM::") + tr("Rename..."));
+                        } else {
+                            // Local-only conflict file
+                            listener->sendMessage(QLatin1String("MENU_ITEM:MOVE_ITEM::") + tr("Rename and upload..."));
+                        }
+                    } else {
+                        if (isOnTheServer) {
+                            // Uploaded conflict file in read-only directory
+                            listener->sendMessage(QLatin1String("MENU_ITEM:MOVE_ITEM::") + tr("Move and rename..."));
+                        } else {
+                            // Local-only conflict file in a read-only dir
+                            listener->sendMessage(QLatin1String("MENU_ITEM:MOVE_ITEM::") + tr("Move, rename and upload..."));
+                        }
+                    }
+                    listener->sendMessage(QLatin1String("MENU_ITEM:DELETE_ITEM::") + tr("Delete local changes"));
+                }
+
+                // File in a read-only directory?
+                if (!isConflict && !isOnTheServer && !canAddToDir) {
+                    listener->sendMessage(QLatin1String("MENU_ITEM:MOVE_ITEM::") + tr("Move and upload..."));
+                    listener->sendMessage(QLatin1String("MENU_ITEM:DELETE_ITEM::") + tr("Delete"));
+                }
+            }
         }
     }
 
-    // Virtual file download action
-    if (folder) {
-        auto virtualFileSuffix = QStringLiteral(APPLICATION_DOTVIRTUALFILE_SUFFIX);
-        bool hasVirtualFile = false;
-        bool hasNormalFiles = false;
-        bool hasDir = false;
+    // File availability actions
+    if (folder
+        && folder->supportsVirtualFiles()
+        && folder->vfs().socketApiPinStateActionsShown()) {
+        ENFORCE(!files.isEmpty());
+
+        // Determine the combined availability status of the files
+        auto combined = Optional<VfsItemAvailability>();
+        auto merge = [](VfsItemAvailability lhs, VfsItemAvailability rhs) {
+            if (lhs == rhs)
+                return lhs;
+            if (int(lhs) > int(rhs))
+                std::swap(lhs, rhs); // reduce cases ensuring lhs < rhs
+            if (lhs == VfsItemAvailability::AlwaysLocal && rhs == VfsItemAvailability::AllHydrated)
+                return VfsItemAvailability::AllHydrated;
+            if (lhs == VfsItemAvailability::AllDehydrated && rhs == VfsItemAvailability::OnlineOnly)
+                return VfsItemAvailability::AllDehydrated;
+            return VfsItemAvailability::Mixed;
+        };
         for (const auto &file : files) {
-            if (QFileInfo(file).isDir()) {
-                hasDir = true;
-            } else if (file.endsWith(virtualFileSuffix)) {
-                hasVirtualFile = true;
-            } else if (!hasNormalFiles) {
-                bool isOnTheServer = FileData::get(file).journalRecord().isValid();
-                hasNormalFiles = isOnTheServer;
+            auto fileData = FileData::get(file);
+            auto availability = folder->vfs().availability(fileData.folderRelativePath);
+            if (!availability) {
+                if (availability.error() == Vfs::AvailabilityError::DbError)
+                    availability = VfsItemAvailability::Mixed;
+                if (availability.error() == Vfs::AvailabilityError::NoSuchItem)
+                    continue;
+            }
+            if (!combined) {
+                combined = *availability;
+            } else {
+                combined = merge(*combined, *availability);
             }
         }
-        if (hasVirtualFile || (hasDir && folder->useVirtualFiles()))
-            listener->sendMessage(QLatin1String("MENU_ITEM:DOWNLOAD_VIRTUAL_FILE::") + tr("Download file(s)", "", files.size()));
 
-        if ((hasNormalFiles || hasDir) && folder->useVirtualFiles())
-            listener->sendMessage(QLatin1String("MENU_ITEM:REPLACE_VIRTUAL_FILE::") + tr("Replace file(s) by virtual file", "", files.size()));
+        // TODO: Should be a submenu, should use icons
+        auto makePinContextMenu = [&](bool makeAvailableLocally, bool freeSpace) {
+            listener->sendMessage(QLatin1String("MENU_ITEM:CURRENT_PIN:d:")
+                                  + Utility::vfsCurrentAvailabilityText(*combined));
+            listener->sendMessage(QLatin1String("MENU_ITEM:MAKE_AVAILABLE_LOCALLY:")
+                                  + (makeAvailableLocally ? QLatin1String(":") : QLatin1String("d:"))
+                                  + Utility::vfsPinActionText());
+            listener->sendMessage(QLatin1String("MENU_ITEM:MAKE_ONLINE_ONLY:")
+                                  + (freeSpace ? QLatin1String(":") : QLatin1String("d:"))
+                                  + Utility::vfsFreeSpaceActionText());
+        };
+
+        if (combined) {
+            switch (*combined) {
+            case VfsItemAvailability::AlwaysLocal:
+                makePinContextMenu(false, true);
+                break;
+            case VfsItemAvailability::AllHydrated:
+            case VfsItemAvailability::Mixed:
+                makePinContextMenu(true, true);
+                break;
+            case VfsItemAvailability::AllDehydrated:
+            case VfsItemAvailability::OnlineOnly:
+                makePinContextMenu(true, false);
+                break;
+            }
+        }
     }
 
     listener->sendMessage(QString("GET_MENU_ITEMS:END"));
